def validate_P5MIN_inputs():
    """
    .. todo:: Create `P5MIN` validator
    """
    return None


def validate_PREDISPATCH_inputs():
    """
    .. todo:: Create `PREDISPATCH` validator
    """
    return None


def validate_PDPASA_inputs():
    """
    .. todo:: Create `PDPASA` validator
    """
    return None


def validate_STPASA_inputs():
    """
    .. todo:: Create `STPASA` validator
    """
    return None


def validate_MTPASA_inputs():
    """
    .. todo:: Create `MTPASA` validator
    .. todo:: Handle MTPASA DUID Availability
    """
    return None
